class ConferencesController < ApplicationController
  before_action :authenticate_user!, except: [ :index, :show, :stats ]
  before_action :authorize_permission, only: [ :new, :create, :edit, :update, :destroy ]
  before_action :set_conference, only: [ :show, :edit, :update, :destroy, :stats ]

  private

  def authorize_permission
    super("manage_conferences")
  end

  def set_conference
    @conference = Conference.find_by(slug: params[:slug])
  end

  def conference_params
    all_params = params.require(:conference).permit(:name, :slug, :starts_at, :ends_at, :url, :time_zone, :import_job_class, data: {}).to_h

    data_hash = @conference&.data&.dup || {}

    if params[:data].present?
      params[:data].each do |key, value|
        data_hash[key] = value.presence
      end
    end

    if params[:custom_field_keys].present? && params[:custom_field_values].present?
      keys = params[:custom_field_keys]
      values = params[:custom_field_values]

      keys.each_with_index do |key, index|
        next if key.blank?
        data_hash[key] = values[index].presence
      end
    end

    all_params[:data] = data_hash

    all_params
  end

  public

  def self.available_import_job_classes
    job_classes = {}

    Rails.application.eager_load! if Rails.env.development?

    ApplicationJob.descendants.each do |job_class|
      if job_class.name.end_with?("ImportJob")
        display_name = job_class.name.demodulize.gsub("ImportJob", "")

        # If the class is in a module, add the module name
        if job_class.name.include?("::")
          module_name = job_class.name.deconstantize
          display_name = "#{module_name} #{display_name}"
        end

        display_name = display_name.gsub("::", " ").gsub(/([A-Z])/, ' \1').strip

        job_classes[job_class.name] = display_name
      end
    end

    job_classes
  end

  def required_fields
    import_job_class = params[:import_job_class]
    result = {
      required_fields: [],
      metadata: {}
    }

    if import_job_class.present?
      begin
        klass = import_job_class.constantize
        result[:required_fields] = klass.respond_to?(:required_data_fields) ? klass.required_data_fields : []
        result[:metadata] = klass.respond_to?(:field_metadata) ? klass.field_metadata : {}
      rescue NameError => e
        Rails.logger.error("Could not find import job class #{import_job_class}: #{e}")
      end
    end

    render json: result
  end

  def index
    @conferences = Conference.all
  end

  def show
    @conference = Conference.find_by(slug: params[:slug])
    @sessions = @conference.sessions.where.not(starts_at: nil).includes(:stage,
                                                                        :assignments).where(stage: @conference.relevant_stages).order(:starts_at)
    if params[:date]
      date = Time.parse(params[:date])
      logger.debug(date)
      start_of_day = lambda { |date|
        date.change(
          hour: Rails.configuration.start_of_day.hour,
          min: Rails.configuration.start_of_day.min
        )
      }
      @sessions = @sessions.where(starts_at: start_of_day.call(date)..start_of_day.call(date.advance(days: 1)))
    end
    @users = User.all
  end

  def new
    @conference = Conference.new
  end

  def create
    @conference = Conference.new(conference_params)

    if @conference.save
      redirect_to conference_path(slug: @conference.slug), notice: "Conference was successfully created."
    else
      render :new, status: :unprocessable_entity
    end
  end

  def edit
    # @conference is set by the before_action
  end

  def update
    if @conference.update(conference_params)
      redirect_to conference_path(slug: @conference.slug), notice: "Conference was successfully updated."
    else
      render :edit, status: :unprocessable_entity
    end
  end

  def destroy
    @conference.destroy
    redirect_to conferences_path, notice: "Conference was successfully deleted."
  end

  def stats
    @conference = Conference.find_by(slug: params[:slug])
    @relevant_stages = @conference.relevant_stages
    @relevant_sessions = @conference.sessions.includes(:stage, :assignments, :speakers,
                                                       assignments: :user).where(stage: @conference.relevant_stages).order(:starts_at)
    @assignees = @relevant_sessions.map(&:assignments).flatten.map(&:user).uniq
    @language_stats = @relevant_sessions.group_by { |s| s.language }.transform_values(&:count)
    @speakers = @relevant_sessions.map(&:speakers)
    @unique_speakers = @speakers.flatten.uniq.count
    @speakers_per_session = @speakers.map(&:count).sum.to_f / @speakers.size.to_f
    @day_stats = @relevant_sessions.group_by { |s| s.starts_at.to_date }.transform_values do |se|
      {
        starts_at: se.first.starts_at,
        ends_at: se.last.ends_at,
        sessions_count: se.count,
        wall_clock_time_min: (se.last.ends_at - se.first.starts_at) / 60.0,
        scheduled_time_min: se.map { |x| (x.ends_at - x.starts_at) / 60.0 }.sum
      }
    end
    @total_stats = @day_stats.values.inject do |m, x|
      m.merge(x) do |_k, o, n|
        o + n
      end
    end.slice(:sessions_count, :wall_clock_time_min, :scheduled_time_min)
  end
end
